/**
	Klasse zur Definition von ganzen Hexadezimalzahlen. Alle Berechnungen 
	werden allerdings im Raum der ganzen Dezimalzahlen ausgefhrt. Dazu
	werden diverse Konvertierungs- und Berechnungsfunktionen bereitgestellt.
	Alle verwendeten Zahlen liegen in Zweierkomplementdarstellung vor.
*/

public class WholeHexNumber {

	/**
		Konstruktor prft, ob bergebener String wirklich Hexzahl
		darstellt. Falls dies der Fall ist, wird eine neue Hexzahl
		angelegt.
		@param number: Stringreprsentation der gewnschten Hex-Nr.
		@throws HexNumberException falls es sich bei der bergebenen
		Nummer nicht um eine gltige Hex-Zahl handelt
	*/
	public WholeHexNumber(String number) throws NotAHexNumberException {

		StringBuffer examCopy = new StringBuffer();
		if (number.length() == 0) {				//	Bei leerem String wird  
			whn = new StringBuffer("0");		//	Null (Hex) zurckgegeben 
			while (whn.length() < 4) {
				whn.insert(0,'0');
			}
		}	
		else {													//	Sonst wird Hexzahl erzeugt
			for (int i = 0; i < number.length(); i++) {
				char c = number.charAt(i);
				switch (c) {
					case '0': 		examCopy.append(c); break;
					case '1': 		examCopy.append(c); break;
					case '2': 		examCopy.append(c); break;
					case '3': 		examCopy.append(c); break;
					case '4': 		examCopy.append(c); break;
					case '5': 		examCopy.append(c); break;
					case '6': 		examCopy.append(c); break;
					case '7': 		examCopy.append(c); break;
					case '8': 		examCopy.append(c); break;
					case '9': 		examCopy.append(c); break;
					case 'a': 		examCopy.append(c); break;
					case 'A': 		examCopy.append('a'); break;
					case 'b': 		examCopy.append(c); break;
					case 'B': 		examCopy.append('b'); break;
					case 'c': 		examCopy.append(c); break;
					case 'C': 		examCopy.append('c'); break;
					case 'd': 		examCopy.append(c); break;
					case 'D': 		examCopy.append('d'); break;
					case 'e': 		examCopy.append(c); break;
					case 'E': 		examCopy.append('e'); break;
					case 'f': 		examCopy.append(c); break;
					case 'F': 		examCopy.append('f'); break;
					default:		break;
				}
			}																				//	Falls Fehler in der Zeichenkette auf-
			if (examCopy.length() == number.length()) {	//	traten sind Stringlnge und Hexzahl-
				whn = new StringBuffer(number);					//	lnge unterschiedlich

				while (whn.length() < 4) {								//	erzeugen von fhrenden Nullen zur	
					whn.insert(0,'0');											//	Normierung der Lnge auf vier
				}
			}
 			else throw new NotAHexNumberException();
		}
	}


	/**
		Diese Methode prft, ob die Hexzahl negativ (hchstes Bit der
		binren Darstellung gesetzt) ist, und gibt true zurck falls das
		der Fall ist
		@return true, falls die Hexadezimalzahl negativ ist, sonst
		false
	*/
	private boolean negHexNumber()	{

		boolean neg = false; 
		if ((whn.charAt(0) == '8') || (whn.charAt(0) == '9') ||
			(whn.charAt(0) == 'a') || (whn.charAt(0) == 'b') ||
			(whn.charAt(0) == 'c') || (whn.charAt(0) == 'd') ||
			(whn.charAt(0) == 'e') || (whn.charAt(0) == 'f')) neg = true;
		return neg;
	}


	/**
		Ziel dieser Methode ist die Umwandlung einer Hexzahl in eine
		Integerzahl (Dezimalzahl).
		@return Dezimaldarstellung der Hexadezimalzahl
	*/
	public int toInteger() {

		int convert = 0;
		int base = 1;
		for (int i = (whn.length()-1); i >= 0; i--) {
			convert += WholeHexNumber.getIntFigure(whn.charAt(i)) * base;
			base *= 16;
		}
		if (negHexNumber()) {
			convert = 65536 - convert;
			convert *= -1;
		}
		return convert;
	}


	/**
		Hier findet die Umwandlung der bergebenen Dezimalzahl in eine
		Hexadezimalzahl statt.
		@param n: umzuwandelnde Dezimalzahl
		@return Hexdarstellung von n
		@throws HexNumberException falls es sich bei der bergebenen
		Nummer nicht um eine gltige Hex-Zahl handelt
	*/ 
	public static WholeHexNumber toHex(int n) {

		StringBuffer convert = new StringBuffer();
		WholeHexNumber whn = null;
		int	mod_int = 0, base = 16;
		if (n < 0) {
				n *= -1;
				n = 65536 - n;
		}
		if (n == 0) convert.append('0');
		else {
			while (n > 0) {
				mod_int = n % base;
				convert.insert(0,	WholeHexNumber.getHexFigure(mod_int));
				n = n / base;
			}
		}
		while (convert.length() < 4) {
			convert.insert(0,'0');
		}
		try {	
			whn = new WholeHexNumber(convert.toString());
		}	catch (NotAHexNumberException e) {
			System.out.println(e.getMessage());
		}
		return whn;
	}				


	/**
		Diese Mehthode kann eine Hexziffer in die ihr entsprechende
		Dezimalzahl umwandeln und diese zurckgeben. Ungltige
		Hex-Ziffern werden als 0 interpretiert.
		@param c: Umzuwandelnde Hex-Ziffer (Achtung: NICHT
		<tt>ZAHL</tt>, sondern nur <tt>ZIFFER</tt>)
		@return Dezimaldarstellung von c
	*/
	private static int getIntFigure(char c) {

		int figure = 0;
		switch (c) {
			case '0':		figure = 0; break;
			case '1':		figure = 1; break;
			case '2':		figure = 2; break;
			case '3':		figure = 3; break;
			case '4':		figure = 4; break;
			case '5':		figure = 5; break;
			case '6':		figure = 6; break;
			case '7':		figure = 7; break;
			case '8':		figure = 8; break;
			case '9':		figure = 9; break;
			case 'a':		figure = 10; break;
			case 'A':		figure = 10; break;
			case 'b':		figure = 11; break;
			case 'B':		figure = 11; break;
			case 'c':		figure = 12; break;
			case 'C':		figure = 12; break;
			case 'd':		figure = 13; break;
			case 'D':		figure = 13; break;
			case 'e':		figure = 14; break;
			case 'E':		figure = 14; break;
			case 'f':		figure = 15; break;
			case 'F':		figure = 15; break;
		}
		return figure;
	}

	
	/**
		Diese Methode kann eine Dezimalzahl in die ihr entsprechende
		Hexadezimalziffer umwandeln und diese zurckgeben. Ungltige
		Dezimalzahlen werden als Leerzeichen zurckgegeben.
		@param figure: Umzuwandelnde Dezimalzahl
		@return Hexadezimaldarstellung von figure, falls sich figure
		als Hexadezimalziffer darstellen lt, Leerzeichen sonst
	*/
	private static char getHexFigure(int figure) {

		char c = ' ';
		switch (figure) {
			case 0:		c = '0'; break;
			case 1:		c = '1'; break;
			case 2:		c = '2'; break;
			case 3:		c = '3'; break;
			case 4:		c = '4'; break;
			case 5:		c = '5'; break;
			case 6:		c = '6'; break;
			case 7:		c = '7'; break;
			case 8:		c = '8'; break;
			case 9:		c = '9'; break;
			case 10:	c = 'a'; break;
			case 11:	c = 'b'; break;
			case 12:	c = 'c'; break;
			case 13:	c = 'd'; break;
			case 14:	c = 'e'; break;
			case 15:	c = 'f'; break;
		}
		return c;
	}
				 

	/**
		Methode zur Ausgabe der Hexadezimalzahl als String.
		@return Hexadezimalzahl als String
	*/
	public String toString() {
		return (whn.toString());
	}


	/**
		Methode zum Bilden der logischen OR Verknpfung zweier Hexadezimalzahlen.
		@param whn1: 1. Hexzahl
		@param	whn2: 2. Hexzahl
		@return Hexadezimaldarstellung des Ergebnisses
	*/
	public static WholeHexNumber HexORHex(WholeHexNumber whn1, WholeHexNumber whn2) {

		int 	whn1_int = 0,
			whn2_int = 0,
			OR_int = 0;
		whn1_int = whn1.toInteger();
		whn2_int = whn2.toInteger();
		OR_int = whn1_int | whn2_int;
		return (WholeHexNumber.toHex(OR_int));
	}

	
	/**
		Methode zum Bilden der logischen XOR Verknpfung zweier Hexadezimalzahlen.
		@param whn1: 1. Hexzahl
		@param	whn2: 2. Hexzahl
		@return Hexadezimaldarstellung des Ergebnisses
	*/
	public static WholeHexNumber HexXORHex(WholeHexNumber whn1, WholeHexNumber whn2) {

		int 	whn1_int = 0,
			whn2_int = 0,
			XOR_int = 0;
		whn1_int = whn1.toInteger();
		whn2_int = whn2.toInteger();
		XOR_int = whn1_int ^ whn2_int;
		return (WholeHexNumber.toHex(XOR_int));
	}


	/**
		Methode zum Bilden der logischen AND Verknpfung zweier Hexadezimalzahlen.
		@param whn1: 1. Hexzahl
		@param	whn2: 2. Hexzahl
		@return Hexadezimaldarstellung des Ergebnisses
	*/
	public static WholeHexNumber HexANDHex(WholeHexNumber whn1, WholeHexNumber whn2) {

		int 	whn1_int = 0,
			whn2_int = 0,
			AND_int = 0;
		whn1_int = whn1.toInteger();
		whn2_int = whn2.toInteger();
		
		AND_int = whn1_int & whn2_int;
		return (WholeHexNumber.toHex(AND_int));
	}


	/**
		Methode zum Bilden der logischen NOT Verknpfung einer Hexadezimalzahl.
		@param whn1: 1. Hexzahl
		@return Hexadezimaldarstellung des Ergebnisses
	*/
	public static WholeHexNumber NOTHex(WholeHexNumber whn) {

		int 	whn_int = whn.toInteger();
		whn_int = ~whn_int;
		return (WholeHexNumber.toHex(whn_int));
	}

	
	/**
		Methode zum Erhhen einer Hexadezimalzahl um 1.
		@param whn1: 1. Hexzahl
		@return Hexadezimaldarstellung des Ergebnisses
	*/
	public static WholeHexNumber INCHex(WholeHexNumber whn) {

		int 	whn_int = whn.toInteger();
		whn_int++;
		return (WholeHexNumber.toHex(whn_int));
	}


	/**
		Methode zum Erniedrigen einer Hexadezimalzahl um 1.
		@param whn1: 1. Hexzahl
		@return Hexadezimaldarstellung des Ergebnisses
	*/
	public static WholeHexNumber DECHex(WholeHexNumber whn) {

		int 	whn_int = whn.toInteger();
		whn_int--;
		return (WholeHexNumber.toHex(whn_int));
	}


	/**
		Methode zum Addieren zweier Hexadezimalzahlen.
		@param whn1: 1. Hexzahl
		@param	whn2: 2. Hexzahl
		@return Hexadezimaldarstellung des Ergebnisses
	*/
	public static WholeHexNumber HexADDHex(WholeHexNumber whn1, WholeHexNumber whn2) {

		int 	whn1_int = 0,
				whn2_int = 0,
				ADD_int = 0;
		whn1_int = whn1.toInteger();
		whn2_int = whn2.toInteger();
		ADD_int = whn1_int + whn2_int;
		return (WholeHexNumber.toHex(ADD_int));
	}


	/**
		Methode zum Subtrahieren zweier Hexadezimalzahlen.
		@param whn1: 1. Hexzahl
		@param	whn2: 2. Hexzahl
		@return Hexadezimaldarstellung des Ergebnisses
	*/
	public static WholeHexNumber HexSUBHex(WholeHexNumber whn1, WholeHexNumber whn2) {

		int 	whn1_int = 0,
				whn2_int = 0,
				SUB_int = 0;
		whn1_int = whn1.toInteger();
		whn2_int = whn2.toInteger();
		SUB_int = whn1_int - whn2_int;
		return (WholeHexNumber.toHex(SUB_int));
	}


	private StringBuffer whn;			//	interne Darstellung der Hexzahl als String

}